import {Component} from '@angular/core'
import {RecipeService} from '../../services/recipe.service'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {Recipe} from '../../../shared/model/recipe.model'
import {Observable, Subject} from 'rxjs'
import {ErrorModel, ErrorService} from '../../../shared/service/error.service'
import {AlertService} from '../../../shared/service/alert.service'
import {GlobalAlertHandlerComponent} from '../../../shared/components/global-alert-handler/global-alert-handler.component'
import {InventoryService} from '../../../material/service/inventory.service'
import {ConfirmBoxComponent} from '../../../shared/components/confirm-box/confirm-box.component'

@Component({
  selector: 'cre-explore',
  templateUrl: './explore.component.html',
  styleUrls: ['./explore.component.sass']
})
export class ExploreComponent extends ErrorHandlingComponent {
  recipe: Recipe | null
  deductErrorBody = {}
  units$ = new Subject<string>()

  hasModifications = false
  note: string | null
  quantitiesChanges = new Map<number, Map<number, number>>()
  mixesLocationChanges = new Map<number, string>()

  deductedMixId: number | null
  handledErrorModels: ErrorModel[] = [{
    filter: error => error.status === 409,
    consumer: error => this.deductErrorBody = {mix: this.deductedMixId, lowQuantities: error.error.lowQuantities},
    messageProducer: () => 'Certains produit ne sont pas en quantité suffisante dans l\'inventaire'
  }]

  constructor(
    private recipeService: RecipeService,
    private inventoryService: InventoryService,
    private alertService: AlertService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit(): void {
    super.ngOnInit()
    GlobalAlertHandlerComponent.extraTopMarginMultiplier = 2.5

    const id = parseInt(this.activatedRoute.snapshot.paramMap.get('id'))
    this.subscribeEntityById(
      this.recipeService,
      id,
      r => {
        this.recipe = r
        this.note = r.note

        if (this.recipe.mixes.length <= 0 || this.recipe.steps.length <= 0) {
          this.alertService.pushWarning('Cette recette n\'est pas complète')
        }
      },
      '/colors/list'
    )
  }

  ngOnDestroy() {
    super.ngOnDestroy()
    GlobalAlertHandlerComponent.extraTopMarginMultiplier = 0
  }

  changeUnits(unit: string) {
    this.units$.next(unit)
  }

  changeNote(event: any) {
    this.hasModifications = true
    this.note = event.target.value
  }

  changeQuantity(event: { id: number, materialId: number, quantity: number }) {
    if (!this.quantitiesChanges.has(event.id)) {
      this.quantitiesChanges.set(event.id, new Map<number, number>())
    }
    this.quantitiesChanges.get(event.id).set(event.materialId, event.quantity)
  }

  changeMixLocation(event: { id: number, location: string }) {
    this.hasModifications = true
    this.mixesLocationChanges.set(event.id, event.location)
  }

  saveModifications() {
    this.subscribe(
      this.recipeService.saveExplorerModifications(this.recipe.id, this.note, this.mixesLocationChanges),
      () => this.alertService.pushSuccess('Les modifications ont été enregistrées'),
      true
    )
  }

  showDeductMixConfirm(mixId: number, confirmBox: ConfirmBoxComponent) {
    this.deductedMixId = mixId
    confirmBox.show()
  }

  deductMix() {
    const firstMixMaterial = this.recipe.mixes.filter(m => m.id === this.deductedMixId)[0].mixMaterials[0]
    if (this.quantitiesChanges.has(this.deductedMixId) && this.quantitiesChanges.get(this.deductedMixId).has(firstMixMaterial.material.id)) {
      const originalQuantity = firstMixMaterial.quantity
      const currentQuantity = this.quantitiesChanges.get(this.deductedMixId).get(firstMixMaterial.material.id)
      this.subscribeDeductMix(this.inventoryService.deductMixFromQuantities(this.deductedMixId, originalQuantity, currentQuantity))
    } else {
      this.subscribeDeductMix(this.inventoryService.deductMix(this.deductedMixId, 1))
    }
  }

  subscribeDeductMix(observable: Observable<any>) {
    this.subscribe(
      observable,
      () => this.alertService.pushSuccess('Les quantités quantités ont été déduites de l\'inventaire'),
      true
    )
  }
}
